@@ -48,10 +48,8 @@ sub Read
 	::setlocale(::LC_NUMERIC, 'C');
 	my @taglist;
 	my %values;	#results will be put in %values
-	my $estimated;
 	if (my $info=$filetag->{info})	#audio properties
-	{	$estimated=$info->{estimated};
-		if ($info->{estimated} && $findlength!=1) { delete $info->{seconds}; delete $info->{bitrate}; }
+	{	if ($findlength!=1 && $info->{estimated}) { delete $info->{$_} for qw/seconds bitrate estimated/; }
 		$formatstring=~s/{(\w+)}/$info->{$1}/g;
 		$values{filetype}=$formatstring;
 		for my $f (grep $Songs::Def{$_}{audioinfo}, @Songs::Fields)
@@ -116,7 +114,7 @@ sub Read
 	}
 	::setlocale(::LC_NUMERIC, '');
 
-	return \%values,$estimated;
+	return \%values;
 }
 
 sub Write
@@ -299,7 +297,7 @@ INIT
 {
  @Tools=
  (	{ label=> _"Capitalize",		for_all => sub { ucfirst lc $_[0]; }, },
-	{ label=>_"Capitalize each word",	for_all => sub { join ' ',map ucfirst lc, split / /,$_[0]; }, },
+	{ label=>_"Capitalize each word",	for_all => sub { join '',map ucfirst lc, split /(\W+)/,$_[0]; }, },
  );
  @FORMATS=
  (	['%a - %l - %n - %t',	qr/(.+) - (.+) - (\d+) - (.+)$/],
@@ -977,7 +975,7 @@ use Gtk2;
 use base 'Gtk2::SpinButton';
 
 sub new
-{	my ($class,$field,$IDs,$max,$digits) = @_;
+{	my ($class,$field,$IDs,$max,$digits,$specialmode) = @_;
 	$max||=10000000;
 	$digits||=0;
 	my $adj=Gtk2::Adjustment->new(0,0,$max,1,10,0);
@@ -992,6 +990,22 @@ sub new
 	}
 	else { $val=Songs::Get($IDs,$field); }
 	$self->set_value($val);
+
+	if ($specialmode)
+	{	if ($specialmode eq 'nozero')
+		{	$self->signal_connect(output=> \&output_nozero);
+		}
+		elsif ($specialmode eq 'year')
+		{	$self->set_wrap(1);
+			# set to current year when increasing or decreasing value from 0
+			$self->signal_connect(value_changed=>sub
+			{	my $v=$_[0]->get_value;
+				$_[0]->set_value( (localtime)[5]+1900 ) if $v==1 || $v>=$max;
+			});
+			$self->signal_connect(output=> \&output_nozero);
+		}
+	}
+
 	return $self;
 }
 sub get_text
@@ -1010,6 +1024,13 @@ sub tool
 {	&GMB::TagEdit::EntryString::tool;
 }
 
+sub output_nozero
+{	my $v=$_[0]->get_value;
+	return 0 if $v;
+	Gtk2::Entry::set_text($_[0],'');
+	return 1;
+}
+
 package GMB::TagEdit::EntryBoolean;
 use Gtk2;
 use base 'Gtk2::CheckButton';
@@ -1573,7 +1594,8 @@ INIT
 	comments	=> [_"Comments",9,'M'],
 	description	=> [_"Description",9,'M'],
 	genre		=> [_"Genre",8],
-	lyrics		=> [_"Lyrics",14,'M'],
+	lyrics		=> [_"Lyrics",14,'L'],
+	fmps_lyrics	=> [_"Lyrics",14,'L'],
 	author		=> [_"Original Artist",40],
 	metadata_block_picture=> [_"Picture",15,'tCTb'],
   };
@@ -1592,6 +1614,7 @@ INIT
 	publicationright=> [_"Publication right",81],
 	year		=> [_"Year",7],
 	'debut album'	=> [_"Debut Album",8],
+	fmps_lyrics	=> [_"Lyrics",14,'L'],
   };
   my $lyrics3v2_types=
   {	LYR => [_"Lyrics",7,'M'],
@@ -1667,6 +1690,7 @@ INIT
 	u => ['EntryBinary'],	#unknown -> binary
 	f => ['EntryBoolean'],
 	p => ['EntryCover'],
+	L => ['EntryLyrics'],
  );
 
 }
@@ -1774,7 +1798,7 @@ sub addrow
 		$type=$DataType{$type}[0] || 'EntrySimple';
 		my $param=$DataType{$type}[1];
 		if ($key eq '') { ($widget,$label)=EntryDouble->new($value); }
-		else	{ $widget=$type->new($value,$param); $label=Gtk2::Label->new($name); }
+		else	{ $widget=$type->new($value,$param); $label=Gtk2::Label->new($name); $label->set_tooltip_text($key); }
 		$table->attach($label,1,2,$row,$row+1,'shrink','shrink',1,1);
 		$table->attach($widget,2,3,$row,$row+1,['fill','expand'],'shrink',1,1);
 		@Todel=($label);
@@ -2070,6 +2094,11 @@ INIT
 				[_"Name",1,1,2],
 				['',2,0,2],
 			],
+		'com.apple.iTunes----FMPS_Lyrics'=>
+		[		[_"Application",0,1,2],
+				[_"Name",1,1,2],
+				['',2,0,2,'EntryLyrics'],
+		],
 	);
 	$SUBTAGPROP{metadata_block_picture}=$SUBTAGPROP{APIC}; #for vorbis pictures
 }
@@ -2080,7 +2109,8 @@ sub new
 	my $self = bless Gtk2::Frame->new($name), $class;
 	my $table=Gtk2::Table->new(1, 4, 0);
 	$self->add($table);
-	my $prop=$SUBTAGPROP{$realkey || $key};
+	my $prop= $SUBTAGPROP{$key};
+	$prop||= $SUBTAGPROP{$realkey} if $realkey;
 	my $row=0;
 	my $subtag=0;
 	for my $t (split //,$type)
