@@ -1,4 +1,4 @@
-# Copyright (C) 2010 Quentin Sculo <squentin@free.fr> and Simon Steinbeiß <simon.steinbeiss@shimmerproject.org>
+# Copyright (C) 2010-2011 Quentin Sculo <squentin@free.fr> and Simon Steinbeiß <simon.steinbeiss@shimmerproject.org>
 #
 # This file is part of Gmusicbrowser.
 # Gmusicbrowser is free software; you can redistribute it and/or modify
@@ -8,7 +8,7 @@
 =gmbplugin ARTISTINFO
 name	Artistinfo
 title	Artistinfo plugin
-version	0.4
+version	0.5
 author  Simon Steinbeiß <simon.steinbeiss@shimmerproject.org>
 author  Pasi Lallinaho <pasi@shimmerproject.org>
 desc	This plugin retrieves artist-relevant information (biography, upcoming events, similar artists) from last.fm.
@@ -28,9 +28,9 @@ use constant
 
 my %sites =
 (
-	biography => ['http://ws.audioscrobbler.com/2.0/?method=artist.getinfo&artist=%a&api_key=7aa688c2466dc17263847da16f297835&autocorrect=1',_"biography",_"Show artist's biography"],
-	events => ['http://ws.audioscrobbler.com/2.0/?method=artist.getevents&artist=%a&api_key=7aa688c2466dc17263847da16f297835&autocorrect=1',_"events",_"Show artist's upcoming events"],
-	similar => ['http://ws.audioscrobbler.com/2.0/?method=artist.getsimilar&artist=%a&api_key=7aa688c2466dc17263847da16f297835&autocorrect=1&limit=%l',_"similar",_"Show similar artists"]);
+	biography => ['http://ws.audioscrobbler.com/2.0/?method=artist.getinfo&artist=%a&api_key=7aa688c2466dc17263847da16f297835&autocorrect=1',_"Biography",_"Show artist's biography"],
+	events => ['http://ws.audioscrobbler.com/2.0/?method=artist.getevents&artist=%a&api_key=7aa688c2466dc17263847da16f297835&autocorrect=1',_"Events",_"Show artist's upcoming events"],
+	similar => ['http://ws.audioscrobbler.com/2.0/?method=artist.getsimilar&artist=%a&api_key=7aa688c2466dc17263847da16f297835&autocorrect=1&limit=%l',_"Similar",_"Show similar artists"]);
 
 my @External=
 (	['lastfm',	"http://www.last.fm/music/%a",								_"Show Artist page on last.fm"],
@@ -48,6 +48,11 @@ my %menuitem=
 	submenu => sub { CreateSearchMenu( Songs::Gid_to_Get('artist',$_[0]{gid}) );  },			#when menu item selected
 	test => sub {$_[0]{mainfield} eq 'artist'},	#the menu item is displayed if returns true
 );
+my $nowplayingaID;
+my $queuewaiting;
+my %queuemode=
+(	order=>10, icon=>'gtk-refresh',	short=> _"similar-artists",		long=> _"Auto-fill queue with similar artists (from last.fm)",	changed=>\&QAutofillSimilarArtists,	keep=>1,save=>1,autofill=>1,
+);
 
 =dop
 my @similarity=
@@ -57,14 +62,17 @@ my @similarity=
 	['medium',	'0.3',	'#453e45'],
 	['lower',	'0.1',	'#9a9a9a'],
 );
+
 =cut
 # lastfm api key 7aa688c2466dc17263847da16f297835
 # "secret" string: 18cdd008e76705eb5f942892d49a71e2
 
 ::SetDefaultOptions(OPT,PathFile	=> "~/.config/gmusicbrowser/bio/%a",
-			ArtistPicSize	=> 100,
+			ArtistPicSize	=> 70,
 			SimilarLimit	=> 15,
-			SimilarRating	=> 50,
+			SimilarRating	=> 20,
+			SimilarLocal	=> 0,
+			SimilarExcludeSeed => 0,
 			Eventformat	=> '%title at %name<br>%startDate<br>%city (%country)<br><br>',
 			Eventformat_history => ['%title<br>%startDate<br><br>','%title on %startDate<br><br>'],
 );
@@ -82,10 +90,13 @@ my $artistinfowidget=
 sub Start {
 	Layout::RegisterWidget(PluginArtistinfo => $artistinfowidget);
 	push @::cMenuAA,\%menuitem;
+	$::QActions{'autofill-similar-artists'} = \%queuemode; ::Update_QueueActionList();
 }
 sub Stop {
 	Layout::RegisterWidget(PluginArtistinfo => undef);
 	@::cMenuAA=  grep $_!=\%menuitem, @::SongCMenu;
+	delete $::QActions{'autofill-similar-artists'}; ::Update_QueueActionList();
+	$queuewaiting->abort if $queuewaiting; $queuewaiting=undef;
 }
 
 sub new
@@ -97,9 +108,8 @@ sub new
 	my $fontsize=$self->style->font_desc;
 	$self->{fontsize} = $fontsize->get_size / Gtk2::Pango->scale;
 	$self->{artist_esc} = "";
-
 	my $statbox=Gtk2::VBox->new(0,0);
-	my $artistpic = Layout::NewWidget("ArtistPic",{forceratio=>1,maxsize=>$::Options{OPT.'ArtistPicSize'},click1=>\&apiczoom,xalign=>0});
+	my $artistpic = Layout::NewWidget("ArtistPic",{forceratio=>1,minsize=>$::Options{OPT.'ArtistPicSize'},click1=>\&apiczoom,xalign=>0,group=>$options->{group},tip=>_"Click to show fullsize image"});
 	for my $name (qw/Ltitle Lstats/)
 	{	my $l=Gtk2::Label->new('');
 		$self->{$name}=$l;
@@ -108,14 +118,14 @@ sub new
 		$statbox->pack_start($l,0,0,2);
 	}
 	$self->{artistrating} = Gtk2::Image->new;
-	$statbox->pack_start($self->{artistrating},0,0,2);
+	$statbox->pack_start($self->{artistrating},0,0,0);
 	my $stateventbox = Gtk2::EventBox->new;
 	$stateventbox->add($statbox);
 	$stateventbox->{group}= $options->{group};
 	$stateventbox->signal_connect(button_press_event => sub {my ($stateventbox, $event) = @_; return 0 unless $event->button == 3; my $ID=::GetSelID($stateventbox); ::ArtistContextMenu( Songs::Get_gid($ID,'artists'),{ ID=>$ID, self=> $stateventbox, mode => 'S'}) if defined $ID; return 1; } ); # FIXME: do a proper cm
 
 	my $artistbox = Gtk2::HBox->new(0,0);
-	$artistbox->pack_start($artistpic,1,1,0);
+	$artistbox->pack_start($artistpic,0,1,0);
 	$artistbox->pack_start($stateventbox,1,1,0);
 
 	my $textview=Gtk2::TextView->new;
@@ -136,19 +146,26 @@ sub new
 	my $tc_artist=Gtk2::TreeViewColumn->new_with_attributes( _"Artist",Gtk2::CellRendererText->new,markup=>0);
 	$tc_artist->set_sort_column_id(0);
 	$tc_artist->set_expand(1);
+	$tc_artist->set_resizable(1);
 	$treeview->append_column($tc_artist);
+	$treeview->set_has_tooltip(1);
+	$treeview->set_tooltip_text("Middle-click on local artists to set a filter on them, right-click non-local artists to search for them on the web.");
 	my $renderer=Gtk2::CellRendererText->new;
 	my $tc_similar=Gtk2::TreeViewColumn->new_with_attributes( "%",$renderer,text => 1);
 	$tc_similar->set_cell_data_func($renderer, sub { my ($column, $cell, $model, $iter, $func_data) = @_; my $rating = $model->get($iter, 1); $cell->set( text => sprintf '%.1f', $rating ); }, undef); # limit similarity rating to one decimal
 	$tc_similar->set_sort_column_id(1);
 	$tc_similar->set_alignment(1.0);
+	$tc_similar->set_min_width(10);
 	$treeview->append_column($tc_similar);
 	$treeview->set_rules_hint(1);
 	$treeview->signal_connect(button_press_event => \&tv_contextmenu);
 	$treeview->{store}=$store;
 
-	my $togglebox = Gtk2::HBox->new();
-	my $group;
+	my $toolbar=Gtk2::Toolbar->new;
+	$toolbar->set_style( $options->{ToolbarStyle}||'both-horiz' );
+	$toolbar->set_icon_size( $options->{ToolbarSize}||'small-toolbar' );
+	#$toolbar->set_show_arrow(1);
+	my $group; my $menugroup;
 	foreach my $key (sort keys %sites)
 	{	my $item = $sites{$key}[1];
 		$item = Gtk2::RadioButton->new($group,$item);
@@ -157,17 +174,40 @@ sub new
 		$item -> set_relief("none");
 		$item -> set_tooltip_text($sites{$key}[2]);
 		$item->set_active( $key eq $self->{site} );
-		$item->signal_connect('toggled' => sub { &toggled_cb($self,$item,$textview); } );
+		$item->signal_connect('toggled' => sub { my $self=::find_ancestor($_[0],__PACKAGE__); toggled_cb($self,$item,$textview); } );
 		$group = $item -> get_group;
-		$togglebox->pack_start($item,1,0,0);
+		my $toolitem=Gtk2::ToolItem->new;
+		$toolitem->add( $item );
+		$toolitem->set_expand(1);
+		$toolbar->insert($toolitem,-1);
+
+# trying to make the radiobuttons overflowable, but no shared groups for radiobuttons and radiomenuitems (group doesn't seem to work for radiomenuitem at all)
+#		my $menuitem=Gtk2::RadioMenuItem->new($menugroup,$sites{$key}[1]);
+#		$menuitem->set_active( $key eq $self->{site} );
+		#$menuitem->set_group($menugroup);
+#		$menuitem->set_draw_as_radio(1);
+#		$menuitem->{key} = $key;
+#		if ($menuitem->get_active) { warn $menuitem->{key}; }
+#		$menuitem->signal_connect('toggled' => sub { &toggled_cb($self,$menuitem,$textview); } );
+#		$toolitem->set_proxy_menu_item($key,$menuitem);
 	}
-
-	my $refresh =	::NewIconButton('gtk-refresh',undef, sub { SongChanged($self,'1'); } ,"none",_"Refresh");
-	my $savebutton =::NewIconButton('gtk-save',   undef, \&Save_text,  "none",_"Save artist biography");
-
-	$togglebox->pack_start($refresh,0,0,0);
-	if (!$::Options{OPT.'AutoSave'}) { $togglebox->pack_start($savebutton,0,0,0); }
-	$statbox->pack_start($togglebox,0,0,0);
+	for my $button
+	(	[refresh => 'gtk-refresh', sub { my $self=::find_ancestor($_[0],__PACKAGE__); SongChanged($self,'1'); },_"Refresh", _"Refresh",0],
+		[save => 'gtk-save',	\&Save_text,	_"Save",	_"Save artist biography",$::Options{OPT.'AutoSave'}],
+	)
+	{	my ($key,$stock,$cb,$label,$tip,$hide)=@$button;
+		my $item=Gtk2::ToolButton->new_from_stock($stock);
+		$item->signal_connect(clicked => $cb);
+		$item->set_tooltip_text($tip) if $tip;
+		my $menuitem = Gtk2::ImageMenuItem->new ($label);
+		$menuitem->set_image( Gtk2::Image->new_from_stock($stock,'menu') );
+		$item->set_proxy_menu_item($key,$menuitem);
+		$toolbar->insert($item,-1) unless $hide;
+	}
+	my $artistinfobox = Gtk2::VBox->new(0,0);
+	$artistinfobox->pack_start($artistbox,1,1,0);
+	$artistinfobox->pack_start($toolbar,0,0,0);
+	#$statbox->pack_start($toolbar,0,0,0);
 	$self->{buffer}=$textview->get_buffer;
 	$self->{store}=$store;
 
@@ -187,7 +227,7 @@ sub new
 	$self->{sw1} = $sw1;
 	$self->{sw2} = $sw2;
 
-	$self->pack_start($artistbox,0,0,0);
+	$self->pack_start($artistinfobox,0,0,0);
 	$self->pack_start($infobox,1,1,0);
 
 	$self->signal_connect(destroy => \&destroy_event_cb);
@@ -219,7 +259,7 @@ sub prefbox
 	my $entry=::NewPrefEntry(OPT.'PathFile' => _"Load/Save Artist Info in :", width=>50);
 	my $preview= Label::Preview->new(preview => \&filename_preview, event => 'CurSong Option', noescape=>1,wrap=>1);
 	my $autosave=::NewPrefCheckButton(OPT.'AutoSave' => _"Auto-save positive finds", tip=>_"only works when the artist-info tab is displayed");
-	my $picsize=::NewPrefSpinButton(OPT.'ArtistPicSize',50,500, step=>5, page=>10, text1=>_"Artist Picture Size : ", text2=>_"(applied after restart)");
+	my $picsize=::NewPrefSpinButton(OPT.'ArtistPicSize',50,500, step=>5, page=>10, text =>_("Artist Picture Size : %d")._"(applied after restart)");
 	my $eventformat=::NewPrefEntry(OPT.'Eventformat' => _"Enter custom event string :", width=>80, tip => _"Use tags from last.fm's XML event pages with a leading % (e.g. %headliner), furthermore linebreaks '<br>' and any text you'd like to have in between. E.g. '%title taking place at %startDate<br>in %city, %country<br><br>'", history=>OPT.'Eventformat_history');
 	my $eventformat_reset=Gtk2::Button->new(_"reset format");
 	$eventformat_reset->{format_combo}=$eventformat;
@@ -232,16 +272,18 @@ sub prefbox
 	});
 	my $similar_limit=::NewPrefSpinButton(OPT.'SimilarLimit',0,500, step=>1, page=>10, text1=>_"Limit similar artists to the first : ", tip=>_"0 means 'show all'");
 	my $similar_rating=::NewPrefSpinButton(OPT.'SimilarRating',0,100, step=>1, text1=>_"Limit similar artists to a rate of similarity : ", tip=>_"last.fm's similarity categories:\n>90 super\n>70 very high\n>50 high\n>30 medium\n>10 lower");
+	my $similar_local=::NewPrefCheckButton(OPT.'SimilarLocal' => _"Only show similar artists from local library", tip=>_"applied on reload");
+	my $similar_exclude_seed=::NewPrefCheckButton(OPT.'SimilarExcludeSeed' => _"Exclude 'seed'-artist from queue", tip=>_"The artists similar to the 'seed'-artist will be used to populate the queue, but you can decide to exclude the 'seed'-artist him/herself.");
 	my $lastfm=::NewIconButton('plugin-artistinfo-lastfm',undef,sub { ::main::openurl("http://www.last.fm/music/"); },'none',_"Open last.fm website in your browser");
 	my $titlebox=Gtk2::HBox->new(0,0);
 	$titlebox->pack_start($picsize,1,1,0);
 	$titlebox->pack_start($lastfm,0,0,5);
-	my $frame_bio=Gtk2::Frame->new(_"biography");
+	my $frame_bio=Gtk2::Frame->new(_"Biography");
 	$frame_bio->add(::Vpack($entry,$preview,$autosave));
-	my $frame_events=Gtk2::Frame->new(_"events");
+	my $frame_events=Gtk2::Frame->new(_"Events");
 	$frame_events->add(::Hpack($eventformat,$eventformat_reset));
-	my $frame_similar=Gtk2::Frame->new(_"similar artists");
-	$frame_similar->add(::Vpack($similar_limit,$similar_rating));
+	my $frame_similar=Gtk2::Frame->new(_"Similar Artists");
+	$frame_similar->add(::Vpack($similar_limit,$similar_rating,$similar_local,$similar_exclude_seed));
 	$vbox->pack_start($_,::FALSE,::FALSE,5) for $titlebox,$frame_bio,$frame_events,$frame_similar;
 	return $vbox;
 }
@@ -325,15 +367,18 @@ sub apiczoom {
 		$apic->modify_bg('GTK_STATE_SELECTED',Gtk2::Gdk::Color->parse('black'));
 		$apic->add($img);
 		$apic->show_all;
-# use a label instead of a normal menu-item for formatted text
-#		my $item=Gtk2::ImageMenuItem->new;
-#		my $label=Gtk2::Label->new;
-#		$label->set_line_wrap(TRUE);
-#		$label->set_alignment(0,.5);
-#		$label->set_markup( AA::ReplaceFields($key,$format,$field,1) );
-#		$item->add($label);
-#		$menu->attach($item, $colnb, $colnb+1, $row, $row+1); if (++$row>$rows) {$row=0;$colnb++;}
+		my $artist = Songs::Gid_to_Get("artist",$aID);
+		my $item=Gtk2::MenuItem->new;
+		$item->modify_fg('GTK_STATE_SELECTED',Gtk2::Gdk::Color->parse('white'));
+		my $label=Gtk2::Label->new;	# use a label instead of a normal menu-item for formatted text
+		$label->set_line_wrap(1);
+		$label->set_justify('center');
+		$label->set_ellipsize('end');
+		$label->set_markup( "<big><b>$artist</b></big>" );
+		$item->add($label);
+		$item->show_all;
 		$menu->append($apic);
+		$menu->append($item);
 		$menu->popup (undef, undef, undef, undef, $event->button, $event->time);
 		return 1;
 	}
@@ -383,11 +428,11 @@ sub SongChanged
 	my $self=::find_ancestor($widget,__PACKAGE__);
 	my $ID = ::GetSelID($self);
 	$force = 0 unless $force;
-	$self -> ArtistChanged( Songs::Get_gid($ID,'artist'),$force);
+	$self -> ArtistChanged( Songs::Get_gid($ID,'artist'),Songs::Get_gid($ID,'album'),$force);
 }
 
 sub ArtistChanged
-{	my ($self,$aID,$force)=@_;
+{	my ($self,$aID,$albumID,$force)=@_;
 	return unless $self->mapped;
 	return unless defined $aID;
 	$self->cancel;
@@ -395,18 +440,19 @@ sub ArtistChanged
 	$self->{artistratingvalue}= int($rating+0.5);
 	$self->{artistratingrange}=AA::Get("rating:range",'artist',$aID);
 	$self->{artistplaycount}=AA::Get("playcount:sum",'artist',$aID);
+	$self->{albumplaycount}=AA::Get("playcount:sum",'album',$albumID);
 	my $tip = join "\n",	_("Average rating:")	.' '.$self->{artistratingvalue},
 				_("Rating range:")	.' '.$self->{artistratingrange},
-				_("Total playcount:")	.' '.$self->{artistplaycount};
+				_("Artist playcount:")	.' '.$self->{artistplaycount},
+				_("Album playcount:")	.' '.$self->{albumplaycount};
 
 	$self->{artistrating}->set_from_pixbuf(Songs::Stars($self->{artistratingvalue},'rating'));
 	$self->{Ltitle}->set_markup( AA::ReplaceFields($aID,"<big><b>%a</b></big>","artist",1) );
 	$self->{Lstats}->set_markup( AA::ReplaceFields($aID,'%X « %s'."\n<small>%y</small>","artist",1) );
 	for my $name (qw/Ltitle Lstats artistrating/) { $self->{$name}->set_tooltip_text($tip); }
-	my $artist = ::url_escapeall( Songs::Gid_to_Get("artist",$aID) );
-	my $url= $sites{$self->{site}}[SITEURL];
-	$url=~s/%a/$artist/;
-	$url=~s/%l/$::Options{OPT.'SimilarLimit'}/;
+
+	my $url = GetUrl($sites{$self->{site}}[SITEURL],$aID);
+	return unless $url;
 	if (!$self->{url} or $url ne $self->{url} or $force == 1) {
 		$self->{url} = $url;
 		if ($self->{site} eq "biography") { # check for local biography file before loading the page
@@ -423,6 +469,15 @@ sub ArtistChanged
 	}
 }
 
+sub GetUrl
+{	my ($url,$aID) = @_;
+	my $artist = ::url_escapeall( Songs::Gid_to_Get("artist",$aID) );
+	return unless length $artist;
+	$url=~s/%a/$artist/;
+	$url=~s/%l/$::Options{OPT.'SimilarLimit'}/;
+	return $url;
+}
+
 sub load_url
 {	my ($self,$url)=@_;
 	$self->set_buffer(_"Loading...");
@@ -451,20 +506,26 @@ sub loaded
 	my $iter=$buffer->get_start_iter;
 
 	my $fontsize = $self->{fontsize};
+	my $tag_warning = $buffer->create_tag(undef,foreground=>"#bf6161",justification=>'center',underline=>'single');
 	my $tag_extra = $buffer->create_tag(undef,foreground_gdk=>$self->style->text_aa("normal"),justification=>'left');
 	my $tag_noresults=$buffer->create_tag(undef,justification=>'center',font=>$fontsize*2,foreground_gdk=>$self->style->text_aa("normal"));
 	my $tag_header = $buffer->create_tag(undef,justification=>'left',font=>$fontsize+1,weight=>Gtk2::Pango::PANGO_WEIGHT_BOLD);
-	my ($artistinfo_ok,$infoheader);
+	my ($artistinfo_ok,$infoheader,$warning);
 
 	if ($self->{site} eq "biography") {
 		$infoheader = _"Artist Biography";
-		$data =~ m|^.*<url>(.*?)</url>.*<listeners>(.*?)</listeners>.*<playcount>(.*?)</playcount>.*<content><\!\[CDATA\[(.*?)\n.*\]\]></content>|s; # last part of the regexp removes the license-notice (=last line)
-		my $url = $1.'/+wiki/edit';
+		$data =~ m|^.*<name>(.*?)</name>.*<url>(.*?)</url>.*<listeners>(.*?)</listeners>.*<playcount>(.*?)</playcount>.*<content><\!\[CDATA\[(.*?)\n.*\]\]></content>|s; # last part of the regexp removes the license-notice (=last line)
+		my $lfm_artist = $1;
+		my $aID = Songs::Get_gid($::SongID,'artist');
+		my $local_artist = Songs::Gid_to_Get("artist",$aID);
+		if ($lfm_artist ne $local_artist) { $warning = "Redirected to: ".$lfm_artist."\n"; }
+		else { $warning = ""; }
+		my $url = $2.'/+wiki/edit';
 		my $href = $buffer->create_tag(undef,justification=>'left',foreground=>"#4ba3d2",underline=>'single');
 		$href->{url}=$url;
-		my $listeners = $2;
-		my $playcount = $3;
-		$data = $4;
+		my $listeners = $3;
+		my $playcount = $4;
+		$data = $5;
 		for ($data) {
 			s/<br \/>|<\/p>/\n/gi; # never more than one empty line
 			s/\n\n/\n/gi; # never more than one empty line (again)
@@ -477,6 +538,7 @@ sub loaded
 			$buffer->insert_with_tags($iter,$infoheader."\n",$tag_header);
 		} # fallback text if artist-info not found
 		else {	$artistinfo_ok = "1";
+			$buffer->insert_with_tags($iter,$warning,$tag_warning);
 			$buffer->insert_with_tags($iter,$infoheader."\n",$tag_header);
 			$buffer->insert($iter,$data);
 			$buffer->insert_with_tags($iter,"\n\n"._"Edit in the last.fm wiki",$href);
@@ -528,8 +590,11 @@ sub loaded
 				if ($aID) {
 					$stats=AA::ReplaceFields($aID,' <span foreground="'.$fgcolor.'">(%X « %s)</span>',"artist",1);
 					$s_artist{url} = "local";
+					$self->{store}->set($self->{store}->append,0,::PangoEsc($s_artist{name}).$stats,1,$s_artist{match} * 100,2,$s_artist{url},3,$aID,4,$s_artist{name});
+				}
+				elsif ($::Options{OPT.'SimilarLocal'} == 0) {
+					$self->{store}->set($self->{store}->append,0,::PangoEsc($s_artist{name}).$stats,1,$s_artist{match} * 100,2,$s_artist{url},3,$aID,4,$s_artist{name});
 				}
-				$self->{store}->set($self->{store}->append,0,::PangoEsc($s_artist{name}).$stats,1,$s_artist{match} * 100,2,$s_artist{url},3,$aID,4,$s_artist{name});
 			}
 
 		}
@@ -585,4 +650,45 @@ sub Save_text
 	else {::ErrorMessage(::__x(_("Error saving artistbio in '{file}' :\n{error}"), file => $file, error => $!),$win);}
 }
 
+sub QAutofillSimilarArtists
+{	$queuewaiting->abort if $queuewaiting; $queuewaiting=undef;
+	return unless $::QueueAction eq 'autofill-similar-artists';
+	return if $::Options{MaxAutoFill}<=@$::Queue;
+	return unless $::SongID;
+
+	$nowplayingaID = Songs::Get_gid($::SongID,'artist');
+	return unless Songs::Gid_to_Get("artist",$nowplayingaID);
+
+	my $url = GetUrl($sites{similar}[0],$nowplayingaID);
+	return unless $url;
+	$queuewaiting=Simple_http::get_with_cb(url => $url, cb => \&PopulateQueue );
+}
+
+sub PopulateQueue
+{	$queuewaiting=undef;
+	if ( $nowplayingaID != Songs::Get_gid($::SongID,'artist')) { QAutofillSimilarArtists; return; }
+	my $data = $_[0];
+
+	return unless $::QueueAction eq 'autofill-similar-artists'; # re-check queueaction and 
+	my $nb=$::Options{MaxAutoFill}-@$::Queue;
+	return unless $nb>0;
+	my @artist_gids;
+	for my $s_artist (split /<\/artist>/, $data) {
+		my %s_artist;
+		$s_artist{$1} = ::decode_html($2) while $s_artist=~ m#<(\w+)>([^<]*)</\1>#g;
+		next unless $s_artist{name};
+		if ($s_artist{match} >= $::Options{OPT.'SimilarRating'} / 100) {
+			my $aID=Songs::Search_artistid($s_artist{name});
+			push (@artist_gids, $aID) if $aID;
+		}
+	}
+	push (@artist_gids, Songs::Get_gid($::SongID,'artist')) unless $::Options{OPT.'SimilarExcludeSeed'}; # add currently playing artist as well
+
+	my $filter= Filter->newadd(0, map Songs::MakeFilterFromGID("artist",$_), @artist_gids );
+	my $random= Random->new('random:',$filter->filter);
+	my @IDs=$random->Draw($nb,[@$::Queue,$::SongID]); # add queue and current song to blacklist (won't draw)
+	return unless @IDs;
+	$::Queue->Push(\@IDs);
+}
+
 1
