@@ -28,6 +28,7 @@ my %Sites=
 (artist =>
  {	googlei => ['google images',"http://images.google.com/images?q=%s&imgsz=medium|large", \&parse_googlei],
 	lastfm => ['last.fm',"http://www.last.fm/music/%a/+images", \&parse_lastfm],
+	discogs => ['discogs.com', "http://api.discogs.com/search?f=xml&type=artists&q=%a", \&parse_discogs],
  },
  album =>
  {	googlei => [_"google images","http://images.google.com/images?q=%s&imgsz=medium|large&imgar=ns", \&parse_googlei],
@@ -36,6 +37,7 @@ my %Sites=
 	#itunesgrabber => ['itunesgrabber',"http://www.thejosher.net/iTunes/index.php?artist=%a&album=%l", \&parse_itunesgrabber],
 	freecovers => ['freecovers.net', "http://www.freecovers.net/api/search/%s", \&parse_freecovers], #could add /Music+CD but then we'd lose /Soundtrack
 	#rateyourmusic=> ['rateyourmusic.com', "http://rateyourmusic.com/search?searchterm=%s&searchtype=l",\&parse_rateyourmusic], # urls results in "403 Forbidden"
+	discogs => ['discogs.com', "http://api.discogs.com/search?f=xml&type=releases&q=%s", \&parse_discogs],
  },
 );
 
@@ -80,17 +82,12 @@ sub prefbox
 
 sub Fetch
 {	my ($field,$gid,$ID)=@_;
-	unless (defined $ID)
-	{	my $list= AA::GetIDs($field,$gid);
-		$ID=$list->[0];
-		return unless defined $ID;
-	}
 	my $mainfield=Songs::MainField($field);	#'artist' or 'album'
 	my $self=bless Gtk2::Window->new;
 	$self->set_border_width(4);
 	my $Bsearch=::NewIconButton('gtk-find',_"Search");
 	my $Bcur=Gtk2::Button->new($mainfield eq 'artist' ? _"Search for current artist" : _"Search for current album");
-	::set_drag($Bcur, dest =>	[::DRAG_ID, sub { $_[0]->get_toplevel->SearchID($_[2]); }], );
+	::set_drag($Bcur, dest =>	[::DRAG_ID, sub { $_[0]->get_toplevel->SearchID(undef,$_[2]); }], );
 	my $Bclose=Gtk2::Button->new_from_stock('gtk-close');
 	my @entry;
 	push @entry, $self->{"searchentry_$_"}=Gtk2::Entry->new for qw/s a l/;
@@ -118,7 +115,7 @@ sub Fetch
 	$Bstop->signal_connect( clicked => sub {$_[0]->get_toplevel->stop });
 	$Bclose->signal_connect(clicked => sub {$_[0]->get_toplevel->destroy});
 	$Bnext->signal_connect( clicked => sub {$_[0]->get_toplevel->NextPage});
-	$Bcur->signal_connect(clicked =>sub {$_[0]->get_toplevel->SearchID($::SongID)});
+	$Bcur->signal_connect(clicked =>sub {$_[0]->get_toplevel->SearchID(undef,$::SongID)});
 	$self->signal_connect( destroy => \&abort);
 	$self->signal_connect( unrealize => sub {$::Options{OPT.'winsize'}=join ' ',$_[0]->get_size; });
 
@@ -128,7 +125,7 @@ sub Fetch
 	$self->{mainfield}=$mainfield;
 	$self->{field}=$field;
 	$self->{site}=$::Options{OPT.'PictureSite_'.$mainfield};
-	$self->SearchID($ID);
+	$self->SearchID($gid,$ID);
 	$self->UpdateSite;
 }
 
@@ -149,12 +146,23 @@ sub UpdateSite
 }
 
 sub SearchID
-{	my ($self,$ID)=@_;
+{	my ($self,$gid,$ID)=@_;	#only one of $gid and $ID needs to be defined
 	$self=::find_ancestor($_[0],__PACKAGE__);
+	my $field= $self->{field};
+	if (!defined $ID)
+	{	return unless defined $gid;
+		my $list= AA::GetIDs($field,$gid);
+		$ID=$list->[0];
+		return unless defined $ID;
+	}
+	elsif (!defined $gid)
+	{	$gid= Songs::Get_gid($ID,$field);
+		$gid= $gid->[0] if ref $gid;	#for field like artists return an array of values, use first value
+	}
 
-	$self->{gid}= Songs::Get_gid($ID,$self->{field});
+	$self->{gid}= $gid;
 	$self->{dir}= Songs::Get($ID,'path');
-	my $search=my $name= Songs::Get($ID,$self->{field});
+	my $search=my $name= Songs::Gid_to_Get($field,$gid);
 	$search="\"$search\"" unless $search eq '';
 	my $albumname='';
 	my $artistname='';
@@ -286,11 +294,17 @@ sub parse_googlei
 		for my $m (@matches)
 		{	my @fields=split /["\]],["\[]/,$m;
 			next if @fields<3;	#happens if no results
-			my $url=$fields[3];
+			my ($url)= $fields[0]=~m#imgurl\\x3d(http.*?)\\x26#;
+			my $preview=$fields[19];
+			unless ($url)
+			{	next unless $preview=~m/^http/;
+				$url=$preview;
+				$preview='';
+			}
+			s/\\x([0-9A-Fa-f]{2})/chr hex($1)/eg for $url,$preview; #unescape urls
 			my $desc= $fields[6]||''; $desc=~s#\\x([0-9a-f]{2})#chr(hex $1)#gie; $desc=~s#</?b>##g;
 			$desc=Encode::decode('cp1252',$desc); #FIXME not sure of the encoding
 			$desc=::decode_html($desc);
-			my $preview='http://images.google.com/images?q=tbn:'.$fields[2].$url;
 			push @list, {url => $url, previewurl =>$preview, desc => $desc };
 		}
 	}
@@ -302,6 +316,15 @@ sub parse_googlei
 	return \@list,$nexturl;
 }
 
+sub parse_discogs
+{	my $result = $_[0];
+	my @list;
+	while ($result =~ m#<thumb>\s*?(.*?)/(A|R)-(\d+)-(.*?)\s*?</thumb>#g)
+	{	push @list, {url => "$1/$2-$4", previewurl => "$1/$2-$3-$4"};
+	}
+	return \@list;
+}
+
 sub searchresults_cb
 {	my ($self,$result)=@_;
 	$self->{waiting}=undef;
