@@ -47,38 +47,44 @@ my @ContextMenuAppend=
 	},
 );
 
-my %sites=	# id => [name,url,?post?,function]	if the function return 1 => lyrics can be saved
-(	#lyrc	=>	['lyrc','http://lyrc.com.ar/en/tema1en.php','artist=%a&songname=%s'],
-	lyrc	=>	['lyrc','http://lyrc.com.ar/en/tema1en.php?artist=%a&songname=%s',undef,sub
-		{	local $_=$_[0];
-			return -1 if m#<a href=[^>]+add[^>]+>(?:[^<]*</?[b-z]\w*)*[^<]*Add a lyric.(?:[^<]*</?[b-z]\w*)*[^<]*</a>#i;
-			return 1 if s#<a href="\#"[^>]+badsong[^>]+>BADSONG</a>##i;
-			return 0;
-		}],
-	#leoslyrics =>	['leolyrics','http://api.leoslyrics.com/api_search.php?artist=%a&songtitle=%s'],
-	#google	=>	['google','http://www.google.com/search?q="%a"+"%s"'],
-	lyriki	=>	['lyriki','http://lyriki.com/index.php?title=%a:%s',undef,
+my %Sites=	# id => [name,url,?post?,function]	if the function return 1 => lyrics can be saved
+(	#lyrc	=>	['lyrc','http://lyrc.com.ar/en/tema1en.php','artist=%a&songname=%t'],
+	#lyrc	=>	['lyrc','http://lyrc.com.ar/en/tema1en.php?artist=%a&songname=%t',undef,sub
+	#	{	local $_=$_[0];
+	#		return -1 if m#<a href=[^>]+add[^>]+>(?:[^<]*</?[b-z]\w*)*[^<]*Add a lyric.(?:[^<]*</?[b-z]\w*)*[^<]*</a>#i;
+	#		return 1 if s#<a href="\#"[^>]+badsong[^>]+>BADSONG</a>##i;
+	#		return 0;
+	#	}],
+	#leoslyrics =>	['leolyrics','http://api.leoslyrics.com/api_search.php?artist=%a&songtitle=%t'],
+	#google	=>	['google','http://www.google.com/search?q="%a"+"%t"'],
+	lyriki	=>	['lyriki','http://lyriki.com/index.php?title=%a:%t',undef,
 		sub { my $no= $_[0]=~m/<div class="noarticletext">/s; $_[0]=~s/^.*<!--\s*start content\s*-->(.*?)<!--\s*end content\s*-->.*$/$1/s && !$no; }],
-	lyricsplugin => [lyricsplugin => 'http://www.lyricsplugin.com/winamp03/plugin/?title=%s&artist=%a',undef,
-			sub { my $ok=$_[0]=~m#<div id="lyrics">.*\w\n.*\w.*</div>#s; $_[0]=~s/<div id="admin".*$//s if $ok; return $ok; }],
-	lyricssongs =>	['lyrics-songs','http://letras.terra.com.br/winamp.php?musica=%s&artista=%a',undef,
-			sub { my $l=html_extract($_[0],div=>'letra'); my $ref=\$_[0]; $$ref=$l ? $l : $notfound; return !!$l }],
-	lyricwiki =>	[lyricwiki => 'http://lyrics.wikia.com/%a:%s',undef,
+	#lyricsplugin => [lyricsplugin => 'http://www.lyricsplugin.com/winamp03/plugin/?title=%t&artist=%a',undef,
+	#		sub { my $ok=$_[0]=~m#<div id="lyrics">.*\w\n.*\w.*</div>#s; $_[0]=~s/<div id="admin".*$//s if $ok; return $ok; }],
+	lyricssongs =>	['lyrics-songs','http://letras.terra.com.br/winamp.php?musica=%t&artista=%a',undef,
+			sub {	my $l=html_extract($_[0],div=>'letra');
+				$l=~s#<div id="cabecalho">.*?</div>##s if $l; #remove header with title and artist links
+				my $ref=\$_[0];
+				$$ref=$l ? $l : $notfound;
+				return ! !$l
+			}],
+	lyricwiki =>	[lyricwiki => 'http://lyrics.wikia.com/%a:%t',undef,
 			 sub {	return 0,'http://lyrics.wikia.com/'.$1 if $_[0]=~m#<span class="redirectText"><a href="/([^"]+)"#;
 				$_[0]=~s!.*<div class='lyricbox'>.*?((?:&\#\d+;|<br ?/>){5,}).*!$1!s; #keep only the "lyric box"
 				return 0 if $_[0]=~m/&#91;&#46;&#46;&#46;&#93;<br/; # truncated lyrics : "[...]" => not auto-saved
 				return !!$1;
 			}],
-	#lyricwikiapi => [lyricwiki => 'http://lyricwiki.org/api.php?artist=%a&song=%s&fmt=html',undef,
+	#lyricwikiapi => [lyricwiki => 'http://lyricwiki.org/api.php?artist=%a&song=%t&fmt=html',undef,
 	#	sub { $_[0]!~m#<pre>\W*Not found\W*</pre>#s }],
-	#azlyrics => [ azlyrics => 'http://search.azlyrics.com/cgi-bin/azseek.cgi?q="%a"+"%s"'],
+	#azlyrics => [ azlyrics => 'http://search.azlyrics.com/cgi-bin/azseek.cgi?q="%a"+"%t"'],
 	#Lyricsfly ?
+	AUTO	=> [_"Auto",],	#special mode that search multiple sources
 );
 
 $::Options{OPT.'Font'} ||= delete $::Options{OPT.'FontSize'};	#for versions <1.1.6
 
-if (my $site=$::Options{OPT.'LyricSite'}) { delete $::Options{OPT.'LyricSite'} unless exists $sites{$site} } #reset selected site if no longer defined
-::SetDefaultOptions(OPT, Font => 10, PathFile => "~/.lyrics/%a/%t.lyric", LyricSite => 'lyricsplugin');
+if (my $site=$::Options{OPT.'LyricSite'}) { delete $::Options{OPT.'LyricSite'} unless exists $Sites{$site} } #reset selected site if no longer defined
+::SetDefaultOptions(OPT, Font => 10, PathFile => "~/.lyrics/%a/%t.lyric", LyricSite => 'lyricssongs', PreferEmbeddedLyrics=>0);
 
 
 my $lyricswidget=
@@ -148,9 +154,10 @@ sub new
 
 	my $adj= $self->{fontsize_adj}= Gtk2::Adjustment->new(10,4,80,1,5,0);
 	my $zoom=Gtk2::ToolItem->new;
-	$zoom->add( Gtk2::SpinButton->new($adj,1,0) );
+	my $zoom_spin=Gtk2::SpinButton->new($adj,1,0);
+	$zoom->add($zoom_spin);
 	$zoom->set_tooltip_text(_"Font size");
-	my $source=::NewPrefCombo( OPT.'LyricSite', {map {$_=>$sites{$_}[0]} keys %sites} ,cb => \&Refresh_cb, toolitem=> _"Lyrics source");
+	my $source=::NewPrefCombo( OPT.'LyricSite', { map {$_=>$Sites{$_}[0]} keys %Sites} ,cb => \&Refresh_cb, toolitem=> _"Lyrics source");
 	my $scroll=::NewPrefCheckButton( OPT.'AutoScroll', _"Auto-scroll", cb=>\&SetAutoScroll, tip=>_"Scroll with the song", toolitem=>1);
 	$toolbar->insert($_,-1) for $follow,$zoom,$scroll,$source;
 
@@ -162,7 +169,7 @@ sub new
 	$self->{buffer}->signal_connect(modified_changed => sub {$_[1]->set_sensitive($_[0]->get_modified);}, $self->{saveb});
 	$self->{backb}->set_sensitive(0);
 	$self->SetFont;
-	$adj->signal_connect(value_changed=> sub { $self->SetFont($_[0]->get_value) });
+	$zoom_spin->signal_connect(value_changed=> sub { my $self=::find_ancestor($_[0],__PACKAGE__); $self->SetFont($_[0]->get_value) });
 	$self->SetToolbarHide($self->{HideToolbar});
 	$self->SetAutoScroll;
 
@@ -182,12 +189,16 @@ sub cancel
 
 sub prefbox
 {	my $vbox=Gtk2::VBox->new(::FALSE, 2);
-	my $entry=::NewPrefEntry(OPT.'PathFile' => _"Load/Save lyrics in :", width=>30);
+	my $entry=::NewPrefEntry(OPT.'PathFile' => _"Lyrics file :", width=>30);
+_"Lyrics file name format",
 	my $preview= Label::Preview->new(preview => \&filename_preview, event => 'CurSong Option', noescape=>1,wrap=>1);
-	my $autosave=::NewPrefCheckButton(OPT.'AutoSave' => _"Auto-save positive finds", tip=>_"only works with some lyrics source and when the lyrics tab is displayed");
+	my $autosave=::NewPrefCheckButton(OPT.'AutoSave' => _"Auto-save positive finds", tip=>_"only works with some lyrics source and when the lyrics tab is active");
+	my $embed=::NewPrefCombo(OPT.'PreferEmbeddedLyrics', { 0=> _"lyrics file", 1=> _"file tag"},
+		text=>_"Prefered place to load and save lyrics :");
+	my $alwaysload=::NewPrefCheckButton(OPT.'AlwaysLoad' => _"Load lyrics even if lyrics panel is hidden");
 	my $Bopen=Gtk2::Button->new(_"open context window");
 	$Bopen->signal_connect(clicked => sub { ::ContextWindow; });
-	$vbox->pack_start($_,::FALSE,::FALSE,1) for $entry,$preview,$autosave,$Bopen;
+	$vbox->pack_start($_,::FALSE,::FALSE,1) for $embed,$entry,$preview,$autosave,$alwaysload,$Bopen;
 	return $vbox;
 }
 
@@ -261,7 +272,7 @@ sub Refresh_cb
 
 sub SongChanged
 {	my ($self,$ID,$force)=@_;
-	return unless $self->mapped;
+	return unless $self->mapped || $::Options{OPT.'AlwaysLoad'};
 	return unless defined $ID;
 	return if defined $self->{ID} && !$force && ( $ID==$self->{ID} || !$self->{follow} );
 	$self->cancel;	#cancel any lyrics operation in progress on an a previous song
@@ -269,21 +280,38 @@ sub SongChanged
 	$self->{time}=undef;
 
 	if (!$force)
-	{	my $file=::pathfilefromformat( $self->{ID}, $::Options{OPT.'PathFile'}, undef,1 );
-		if ($file && -r $file)
-		{	::IdleDo('8_lyrics'.$self,1000,\&load_file,$self,$file);
-			return
+	{	::IdleDo('8_lyrics'.$self,1000,\&load_from_file,$self);
+	}
+	else { $self->load_from_web; }
+}
+
+sub load_from_web
+{	my ($self,$next)=@_;
+	my $site;
+	if ($next)
+	{	$site = shift @{$self->{trynext}};
+		if (!$site)
+		{	$self->{buffer}->set_text(_"Not found.");
+			$self->{buffer}->set_modified(0);
+			return;
 		}
 	}
-
-	my ($title,$artist)= map ::url_escapeall($_), Songs::Get($ID,qw/title artist/);
-	my (undef,$url,$post,$check)=@{$sites{$::Options{OPT.'LyricSite'}}};
+	else
+	{	$site = $::Options{OPT.'LyricSite'};
+		delete $self->{trynext};
+		if ($site eq 'AUTO')
+		{	($site,@{$self->{trynext}})= qw/lyricssongs lyricwiki/;	#FIXME make it configurable
+		}
+	}
+	return unless $site;
+	my (undef,$url,$post,$check)=@{$Sites{$site}};
+	my $ID= $self->{ID};
 	for my $val ($url,$post)
 	{	next unless defined $val;
-		if (ref $val) { $val= $val->(Songs::Get($ID,qw/title artist/),$ID); }
-		else {	$val=~s/%a/$artist/; $val=~s/%s/$title/; }
+		if (ref $val) { $val= $val->($ID); }
+		else { $val= ::ReplaceFields($ID, $val, \&::url_escapeall); }
 	}
-	#$self->load_url($url,$post);
+	return unless $url;
 	::IdleDo('8_lyrics'.$self,1000,\&load_url,$self,$url,$post,$check);
 }
 
@@ -383,8 +411,9 @@ sub loaded #_very_ crude html to gtktextview renderer
 
 	my $oklyrics;
 	if (my $check=$self->{check})
-	{	($oklyrics,my $redirect)= $check->($data);
+	{	($oklyrics,my $redirect)= $check->($data,$self->{ID});
 		if ($redirect) { $self->load_url($redirect,undef,$check); return; }
+		if ($self->{trynext} && !($oklyrics && $oklyrics>0)) { $self->load_from_web('trynext'); return }
 	}
 	if ($self->{lastokurl})
 	{	my $history=$self->{history}||=[];
@@ -491,9 +520,14 @@ sub loaded #_very_ crude html to gtktextview renderer
 		else {$url=undef}
 	}
 
-	$self->{pixtoload}=\@pixbufs;
-	::IdleDo('8_FetchPix'.$self,100,\&load_pixbuf,$self) if @pixbufs;
-	$self->Save_text if $::Options{OPT.'AutoSave'} && $oklyrics && $oklyrics>0;
+	if (0)
+	{	$self->{pixtoload}=\@pixbufs;
+		::IdleDo('8_FetchPix'.$self,100,\&load_pixbuf,$self) if @pixbufs;
+	}
+	if ($oklyrics && $oklyrics>0)
+	{	$self->Save_text if $::Options{OPT.'AutoSave'};
+	}
+	else { $buffer->set_modified(0); }
 }
 
 sub load_pixbuf
@@ -615,21 +649,26 @@ sub update_cursor_cb
 	$textview->get_window('text')->set_cursor(Gtk2::Gdk::Cursor->new($cursor));
 }
 
-sub load_file
-{	my ($self,$file)=@_;
+sub load_from_file
+{	my $self=shift;
+	my $ID= $self->{ID};
+	my $text= $::Options{OPT.'PreferEmbeddedLyrics'} ?
+		FileTag::GetLyrics($ID) || _load_from_lyrics_file($ID) :
+		_load_from_lyrics_file($ID) || FileTag::GetLyrics($ID) ;
+
+	# if no lyrics found, try the web
+	unless ($text)
+	{	$self->load_from_web;
+		return
+	}
+
+	# display the lyrics
 	my $buffer=$self->{buffer};
 	$buffer->delete($buffer->get_bounds);
-	my $text=_("Loading failed.");
-	if (open my$fh,'<',$file)
-	{	local $/=undef; #slurp mode
-		$text=<$fh>;
-		close $fh;
-		if (my $utf8=Encode::decode_utf8($text)) {$text=$utf8}
-	}
         $buffer->set_text($text);
 
 	#make the title and artist bigger and bold
-	my ($title,$artist)= Songs::Get($self->{ID},qw/title artist/);
+	my ($title,$artist)= Songs::Get($ID,qw/title artist/);
 	$title='' if $title!~m/\w\w/;
 	for ($title,$artist)
 	{	if (m/\w\w/) {s#\W+#\\W*#g}
@@ -644,23 +683,43 @@ sub load_file
 	$buffer->set_modified(0);
 }
 
+sub _load_from_lyrics_file
+{	my $ID=shift;
+	my $file=::pathfilefromformat( $ID, $::Options{OPT.'PathFile'}, undef,1 );
+	return unless $file && -e $file;
+	my $text= _"Loading failed.";
+	if (open my$fh,'<',$file)
+	{	local $/=undef; #slurp mode
+		$text=<$fh>;
+		close $fh;
+		if (my $utf8=Encode::decode_utf8($text)) {$text=$utf8}
+	}
+	return $text;
+}
+
 sub Save_text
 {	my $self=::find_ancestor($_[0],__PACKAGE__);
 	my $win=$self->get_toplevel;
 	my $buffer=$self->{buffer};
 	my $text= $buffer->get_text($buffer->get_bounds, ::FALSE);
-	my $format=$::Options{OPT.'PathFile'};
-	my ($path,$file)=::pathfilefromformat( $self->{ID}, $format, undef,1 );
-	unless ($path && $file) {::ErrorMessage(_("Error: invalid filename pattern")." : $format",$win); return}
-	my $res=::CreateDir($path,$win);
-	return unless $res eq 'ok';
-	if (open my$fh,'>:utf8',$path.::SLASH.$file)
-	{	print $fh $text;
-		close $fh;
+	if ($::Options{OPT.'PreferEmbeddedLyrics'})
+	{	FileTag::WriteLyrics($self->{ID},$text);
 		$buffer->set_modified(0);
-		warn "Saved lyrics in ".$path.::SLASH.$file."\n" if $::debug;
 	}
-	else {::ErrorMessage(::__x(_("Error saving lyrics in '{file}' :\n{error}"), file => $file, error => $!),$win);}
+	else
+	{	my $format=$::Options{OPT.'PathFile'};
+		my ($path,$file)=::pathfilefromformat( $self->{ID}, $format, undef,1 );
+		unless ($path && $file) {::ErrorMessage(_("Error: invalid filename pattern")." : $format",$win); return}
+		my $res=::CreateDir($path,$win);
+		return unless $res eq 'ok';
+		if (open my$fh,'>:utf8',$path.::SLASH.$file)
+		{	print $fh $text;
+			close $fh;
+			$buffer->set_modified(0);
+			warn "Saved lyrics in ".$path.::SLASH.$file."\n" if $::debug;
+		}
+		else {::ErrorMessage(::__x(_("Error saving lyrics in '{file}' :\n{error}"), file => $file, error => $!),$win);}
+	}
 }
 
 1
